# Generated by Django 2.0.1 on 2018-02-08 06:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('jacc', '0010_auto_20180201_0734'),
        ('jbank', '0014_payout_payoutstatus'),
    ]

    operations = [
        migrations.CreateModel(
            name='Refund',
            fields=[
                ('payout_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jbank.Payout')),
            ],
            options={
                'verbose_name_plural': 'refunds',
                'verbose_name': 'refund',
            },
            bases=('jbank.payout',),
        ),
        migrations.RemoveField(
            model_name='refundrecord',
            name='accountentry_ptr',
        ),
        migrations.DeleteModel(
            name='RefundRecord',
        ),
    ]



def list_dir_files(path: str, suffix: str= '') -> list:
    """
    Lists all files (and only files) in a directory, or return [path] if path is a file itself.
    :param path: Directory or a file
    :param suffix: Optional suffix to match (case insensitive). Default is none.
    :return: list of absolute paths to files
    """
    import os
    from pathlib import Path

    if suffix:
        suffix = suffix.lower()
    if Path(path).is_file():
        files = [path]
    else:
        files = []
        for f in os.listdir(path):
            file_path = os.path.join(path, f)
            if Path(file_path).is_file():
                if not suffix or f.lower().endswith(suffix):
                    files.append(file_path)
    return files



import logging
from os.path import basename
from django.core.management import CommandParser
from jbank.files import list_dir_files
from jbank.helpers import process_pain002_file_content
from jbank.models import Payout, PayoutStatus, PAYOUT_PAID
from jbank.sepa import Pain002
from jutil.command import SafeCommand


logger = logging.getLogger(__name__)


class Command(SafeCommand):
    help = 'Parses pain.002 payment response .XP files and updates Payout status'

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('path', type=str)
        parser.add_argument('--test', action='store_true')
        parser.add_argument('--verbose', action='store_true')

    def do(self, *args, **options):
        files = list_dir_files(options['path'], '.XP')
        for f in files:
            if PayoutStatus.objects.is_file_processed(f):
                if options['verbose']:
                    print('Skipping processed payment status file', f)
                continue
            if options['verbose']:
                print('Importing payment status file', f)
            with open(f, 'rb') as fp:
                process_pain002_file_content(fp.read(), f)



# Generated by Django 2.0 on 2018-02-08 11:30

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0017_payoutparty_payouts_account'),
    ]

    operations = [
        migrations.AlterField(
            model_name='payoutparty',
            name='payouts_account',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, to='jacc.Account', verbose_name='payouts account'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-11-02 22:38
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0004_statementfile'),
    ]

    operations = [
        migrations.AddField(
            model_name='statementfile',
            name='created',
            field=models.DateTimeField(blank=True, db_index=True, default=django.utils.timezone.now, verbose_name='created'),
        ),
    ]



import base64
import logging
from os.path import basename
from pprint import pprint

import requests
from django.conf import settings


logger = logging.getLogger(__name__)


def wsedi_get(command: str, file_type: str, status: str, file_reference: str='', verbose: bool=False) -> requests.Response:
    """
    Download Finnish bank files. Assumes WS-EDI API parameter compatible HTTP REST API end-point.
    Uses project settings WSEDI_URL and WSEDI_TOKEN.
    :param command: Command, e.g. DownloadFileList or DownloadFile
    :param file_type: File type, e.g. TO or SVM
    :param status: Status, e.g. DLD or NEW
    :param file_reference: File reference (if command is DownloadFile)
    :param verbose: Debug output
    :return: requests.Response
    """
    url = settings.WSEDI_URL + '?command={command}'.format(command=command)
    if file_reference:
        url += '&file-reference=' + file_reference
    if file_type:
        url += '&file-type=' + file_type
    if status:
        url += '&status=' + status
    headers = {
        'Content-Type': 'application/json',
        'Authorization': 'Token ' + settings.WSEDI_TOKEN,
    }
    res = requests.get(url, headers=headers)
    if res.status_code >= 300:
        logger.error("wsedi_get(command={}, file_type={}, status={}, file_reference={}) response HTTP {}:\n".format(command, file_type, status, file_reference, res.status_code) + res.text)
    elif verbose:
        logger.info("wsedi_get(command={}, file_type={}, status={}, file_reference={}) response HTTP {}:\n".format(command, file_type, status, file_reference, res.status_code) + res.text)
    return res


def wsedi_upload_file(file_content: str, file_type: str, file_name: str, verbose: bool=False) -> requests.Response:
    """
    Upload Finnish bank file. Assumes WS-EDI API parameter compatible HTTP REST API end-point.
    Uses project settings WSEDI_URL and WSEDI_TOKEN.
    :param file_content: File content
    :param file_type: File type, e.g. pain.001.001.03
    :param file_name: File (base) name
    :param verbose: Debug output
    :return: requests.Response
    """
    command = 'UploadFile'
    url = settings.WSEDI_URL
    data = {
        'command': command,
        'file-type': file_type,
        'file-name': basename(file_name),
        'file-content': base64.b64encode(file_content.encode('utf8')).decode('ascii'),
    }
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Token ' + settings.WSEDI_TOKEN,
    }
    res = requests.post(url, data=data, headers=headers)
    if res.status_code >= 300:
        logger.error("wsedi_upload_file(command={}, file_type={}, file_name={}) response HTTP {}:\n".format(command, file_type, file_name, res.status_code) + res.text)
    elif verbose:
        logger.info("wsedi_upload_file(command={}, file_type={}, file_name={}) response HTTP {}:\n".format(command, file_type, file_name, res.status_code) + res.text)
    return res



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-10-25 17:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ReferencePaymentBatch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filename', models.CharField(db_index=True, max_length=64, verbose_name='filename')),
                ('record_date', models.DateTimeField(db_index=True, verbose_name='record date')),
                ('institution_identifier', models.CharField(max_length=2, verbose_name='institution identifier')),
                ('service_identifier', models.CharField(max_length=9, verbose_name='service identifier')),
                ('currency_identifier', models.CharField(choices=[('1', 'EUR')], max_length=3, verbose_name='currency identifier')),
            ],
            options={
                'verbose_name_plural': 'reference payment batches',
                'verbose_name': 'reference payment batch',
            },
        ),
        migrations.CreateModel(
            name='ReferencePaymentRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('record_type', models.CharField(max_length=1, verbose_name='record type')),
                ('account_number', models.CharField(db_index=True, max_length=32, verbose_name='account number')),
                ('record_date', models.DateField(db_index=True, verbose_name='record date')),
                ('paid_date', models.DateField(blank=True, db_index=True, default=None, null=True, verbose_name='paid date')),
                ('archive_identifier', models.CharField(blank=True, db_index=True, default='', max_length=32, verbose_name='archive identifier')),
                ('remittance_info', models.CharField(db_index=True, max_length=32, verbose_name='remittance info')),
                ('payer_name', models.CharField(db_index=True, max_length=12, verbose_name='payer name')),
                ('currency_identifier', models.CharField(choices=[('1', 'EUR')], max_length=1, verbose_name='currency identifier')),
                ('name_source', models.CharField(blank=True, choices=[('', 'Not Set'), ('A', 'From Customer'), ('K', 'From Bank Clerk'), ('J', 'From Bank System')], max_length=1, verbose_name='name source')),
                ('amount', models.DecimalField(db_index=True, decimal_places=2, max_digits=10, verbose_name='amount')),
                ('correction_identifier', models.CharField(choices=[('0', 'Regular Entry'), ('1', 'Correction Entry')], max_length=1, verbose_name='correction identifier')),
                ('delivery_method', models.CharField(choices=[('A', 'From Customer'), ('K', 'From Bank Clerk'), ('J', 'From Bank System')], db_index=True, max_length=1, verbose_name='delivery method')),
                ('receipt_code', models.CharField(blank=True, choices=[('', ''), ('E', 'Separate'), ('P', 'Separate/Paper')], db_index=True, max_length=1, verbose_name='receipt code')),
                ('batch', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='record_set', to='jbank.ReferencePaymentBatch', verbose_name='batch')),
            ],
            options={
                'verbose_name_plural': 'reference payment records',
                'verbose_name': 'reference payment records',
            },
        ),
        migrations.CreateModel(
            name='Statement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filename', models.CharField(db_index=True, max_length=64, verbose_name='filename')),
                ('account_number', models.CharField(db_index=True, max_length=32, verbose_name='account number')),
                ('statement_number', models.SmallIntegerField(verbose_name='statement number')),
                ('begin_date', models.DateField(db_index=True, verbose_name='begin date')),
                ('end_date', models.DateField(db_index=True, verbose_name='end date')),
                ('record_date', models.DateTimeField(db_index=True, verbose_name='record date')),
                ('customer_identifier', models.CharField(max_length=32, verbose_name='customer identifier')),
                ('begin_balance_date', models.DateField(verbose_name='begin balance date')),
                ('begin_balance', models.DecimalField(decimal_places=2, max_digits=10, verbose_name='begin balance')),
                ('record_count', models.IntegerField(default=None, null=True, verbose_name='record count')),
                ('currency_code', models.CharField(max_length=3, verbose_name='currency code')),
                ('account_name', models.CharField(blank=True, default='', max_length=32, verbose_name='account name')),
                ('account_limit', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='account limit')),
                ('owner_name', models.CharField(max_length=64, verbose_name='owner name')),
                ('contact_info_1', models.CharField(max_length=64, verbose_name='contact info (1)')),
                ('contact_info_2', models.CharField(blank=True, default='', max_length=64, verbose_name='contact info (2)')),
                ('bank_specific_info_1', models.CharField(blank=True, default='', max_length=32, verbose_name='bank specific info (1)')),
                ('iban', models.CharField(db_index=True, max_length=32, verbose_name='IBAN')),
                ('bic', models.CharField(db_index=True, max_length=8, verbose_name='BIC')),
            ],
            options={
                'verbose_name_plural': 'statements',
                'verbose_name': 'statement',
            },
        ),
        migrations.CreateModel(
            name='StatementRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('record_number', models.IntegerField(verbose_name='record number')),
                ('archive_identifier', models.CharField(blank=True, db_index=True, default='', max_length=32, verbose_name='archive identifier')),
                ('record_date', models.DateField(db_index=True, verbose_name='record date')),
                ('value_date', models.DateField(blank=True, db_index=True, default=None, null=True, verbose_name='value date')),
                ('paid_date', models.DateField(blank=True, db_index=True, default=None, null=True, verbose_name='paid date')),
                ('entry_type', models.CharField(choices=[('1', 'Deposit'), ('2', 'Withdrawal'), ('3', 'Deposit Correction'), ('4', 'Withdrawal Correction')], db_index=True, max_length=1, verbose_name='entry type')),
                ('record_code', models.CharField(choices=[('700', 'Money Transfer (In/Out)'), ('701', 'Recurring Payment (In/Out)'), ('702', 'Bill Payment (Out)'), ('703', 'Payment Terminal Deposit (In)'), ('704', 'Bank Draft (In/Out)'), ('705', 'Reference Payments (In)'), ('706', 'Payment Service (Out)'), ('710', 'Deposit (In)'), ('720', 'Withdrawal (Out)'), ('721', 'Card Payment (Out)'), ('722', 'Check (Out)'), ('730', 'Bank Fees (Out)'), ('740', 'Interests Charged (Out)'), ('750', 'Interests Credited (In)'), ('760', 'Loan (Out)'), ('761', 'Loan Payment (Out)'), ('770', 'Foreign Transfer (In/Out)'), ('780', 'Zero Balancing (In/Out)'), ('781', 'Sweeping (In/Out)'), ('782', 'Topping (In/Out)')], db_index=True, max_length=3, verbose_name='record type')),
                ('record_description', models.CharField(max_length=64, verbose_name='record description')),
                ('amount', models.DecimalField(db_index=True, decimal_places=2, max_digits=10, verbose_name='amount')),
                ('receipt_code', models.CharField(blank=True, choices=[('', ''), ('E', 'Separate'), ('P', 'Separate/Paper')], db_index=True, max_length=1, verbose_name='receipt code')),
                ('delivery_method', models.CharField(choices=[('A', 'From Customer'), ('K', 'From Bank Clerk'), ('J', 'From Bank System')], db_index=True, max_length=1, verbose_name='delivery method')),
                ('name', models.CharField(blank=True, db_index=True, max_length=64, verbose_name='name')),
                ('name_source', models.CharField(blank=True, choices=[('', 'Not Set'), ('A', 'From Customer'), ('K', 'From Bank Clerk'), ('J', 'From Bank System')], max_length=1, verbose_name='name source')),
                ('recipient_account_number', models.CharField(blank=True, db_index=True, max_length=32, verbose_name='recipient account number')),
                ('recipient_account_number_changed', models.CharField(blank=True, max_length=1, verbose_name='recipient account number changed')),
                ('remittance_info', models.CharField(db_index=True, max_length=32, verbose_name='remittance info')),
                ('messages', models.TextField(blank=True, default='', verbose_name='messages')),
                ('client_messages', models.TextField(blank=True, default='', verbose_name='client messages')),
                ('bank_messages', models.TextField(blank=True, default='', verbose_name='bank messages')),
                ('statement', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='record_set', to='jbank.Statement', verbose_name='statement')),
            ],
            options={
                'verbose_name_plural': 'statement records',
                'verbose_name': 'statement record',
            },
        ),
        migrations.CreateModel(
            name='StatementRecordSepaInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('reference', models.CharField(blank=True, max_length=35, verbose_name='reference')),
                ('iban_account_number', models.CharField(blank=True, max_length=35, verbose_name='IBAN')),
                ('bic_code', models.CharField(blank=True, max_length=35, verbose_name='BIC')),
                ('recipient_name_detail', models.CharField(blank=True, max_length=70, verbose_name='recipient name detail')),
                ('payer_name_detail', models.CharField(blank=True, max_length=70, verbose_name='payer name detail')),
                ('identifier', models.CharField(blank=True, max_length=35, verbose_name='identifier')),
                ('archive_identifier', models.CharField(blank=True, max_length=35, verbose_name='archive identifier')),
                ('record', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='sepa_info', to='jbank.StatementRecord', verbose_name='record')),
            ],
            options={
                'verbose_name_plural': 'SEPA',
                'verbose_name': 'SEPA',
            },
        ),
    ]



# Generated by Django 2.1.2 on 2018-11-01 14:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0024_auto_20180425_1704'),
    ]

    operations = [
        migrations.AddField(
            model_name='payout',
            name='reference',
            field=models.CharField(blank=True, default='', max_length=32, verbose_name='recipient reference'),
        ),
        migrations.AlterField(
            model_name='payout',
            name='messages',
            field=models.TextField(blank=True, default='', verbose_name='recipient messages'),
        ),
    ]



import os
from copy import copy
from pathlib import Path
from pprint import pprint

from django.conf import settings
from django.core.files import File
from django.core.management import CommandParser
from django.db import transaction
from django.utils import translation
from jacc.models import Account
from jbank.helpers import create_statement, get_or_create_bank_account
from jbank.files import list_dir_files
from jbank.models import Statement, StatementFile
from jbank.parsers import parse_tiliote_statements_from_file
from jutil.command import SafeCommand


class Command(SafeCommand):
    help = 'Parses bank account statement .TO (tiliote) files'

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('path', type=str)
        parser.add_argument('--verbose', action='store_true')
        parser.add_argument('--test', action='store_true')
        parser.add_argument('--delete-old', action='store_true')
        parser.add_argument('--auto-create-accounts', action='store_true')

    def do(self, *args, **options):
        files = list_dir_files(options['path'])
        for filename in files:
            plain_filename = os.path.basename(filename)
            if options['delete_old']:
                Statement.objects.filter(name=plain_filename).delete()

            if options['test']:
                statements = parse_tiliote_statements_from_file(filename)
                pprint(statements)
                continue

            if not Statement.objects.filter(name=plain_filename).first():
                print('Importing statement file {}'.format(plain_filename))

                statements = parse_tiliote_statements_from_file(filename)
                if options['verbose']:
                    pprint(statements)

                with transaction.atomic():
                    if not Statement.objects.filter(name=plain_filename).first():
                        file = StatementFile()
                        file.save()
                        with open(filename, 'rb') as fp:
                            file.file.save(plain_filename, File(fp))
                        for data in statements:
                            if options['auto_create_accounts']:
                                account_number = data.get('header', {}).get('account_number')
                                if account_number:
                                    get_or_create_bank_account(account_number)

                            create_statement(data, name=plain_filename, file=file)
            else:
                print('Skipping statement file {}'.format(filename))



import subprocess
from datetime import date

from decimal import Decimal
from os.path import join
from pprint import pprint

from django.conf import settings
from django.test import TestCase
from jbank.parsers import parse_tiliote_statements_from_file, parse_svm_batches_from_file
from jbank.sepa import Pain001, Pain002, PAIN001_REMITTANCE_INFO_OCR, PAIN001_REMITTANCE_INFO_OCR_ISO
from jutil.format import format_xml
from jutil.validators import iban_bic


class Tests(TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_pain001(self):
        debtor_acc = 'FI4947300010416310'
        p = Pain001('201802071211XJANITEST', 'Vuokrahelppi', debtor_acc, iban_bic(debtor_acc), '020840699',
                    ['Koukkukankareentie 29', '20320 Turku'], 'FI')
        creditor_acc = 'FI8847304720017517'
        p.add_payment('201802071339A0001', 'Jani Kajala', creditor_acc, iban_bic(creditor_acc), Decimal('49.00'), 'vuokratilitys')
        p.add_payment('201802071339A0001', 'Jani Kajala', creditor_acc, iban_bic(creditor_acc), Decimal('49.00'), '302300', PAIN001_REMITTANCE_INFO_OCR)
        p.add_payment('201802071339A0001', 'Jani Kajala', creditor_acc, iban_bic(creditor_acc), Decimal('49.00'), 'RF92 1229', PAIN001_REMITTANCE_INFO_OCR_ISO)
        xml_str = format_xml(p.render().decode())
        # print(xml_str)

        filename = '/tmp/pain001.xml'
        with open(filename, 'wt') as fp:
            fp.write(xml_str)
            # print(filename, 'written')

        # /usr/bin/xmllint --format --pretty 1 --load-trace --debug --schema $1 $2
        res = subprocess.run([
            '/usr/bin/xmllint',
            '--noout',
            # '--format',
            # '--pretty', '1',
            # '--load-trace',
            # '--debug',
            '--schema',
            join(settings.BASE_DIR, 'data/pain001/pain.001.001.03.xsd'),
            filename,
        ])
        self.assertEqual(res.returncode, 0)

    def test_to(self):
        filename = join(settings.BASE_DIR, 'data/to/547404896.TO')
        statements = parse_tiliote_statements_from_file(filename)
        rec = statements[0]['records'][0]
        # pprint(rec)
        self.assertEqual(rec['amount'], Decimal('-1799.00'))
        self.assertEqual(rec['archive_identifier'], '180203473047IE5807')
        self.assertEqual(rec['paid_date'], date(2018, 2, 3))
        self.assertEqual(rec['sepa']['iban_account_number'], 'FI8847304720017517')

    def test_svm(self):
        filename = join(settings.BASE_DIR, 'data/svm/547392460.SVM')
        batches = parse_svm_batches_from_file(filename)
        recs = batches[0]['records']
        self.assertEqual(len(recs), 1)
        rec = recs[0]
        # pprint(rec)
        self.assertEqual(rec['amount'], Decimal('49.00'))
        self.assertEqual(rec['archive_identifier'], '02042588WWRV0212')
        self.assertEqual(rec['remittance_info'], '00000000000000013013')

    def test_xp(self):
        filename = join(settings.BASE_DIR, 'data/xp/547958656.XP')
        with open(filename, 'rb') as fp:
            file_content = fp.read()
        p = Pain002(file_content)
        self.assertEqual(p.original_msg_id, '201802071211XJANITEST')
        self.assertEqual(p.msg_id, 'V000000009726773')
        self.assertEqual(p.group_status, 'ACCP')
        self.assertEqual(p.is_accepted, True)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-11-03 00:07
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0007_auto_20171102_2351'),
    ]

    operations = [
        migrations.CreateModel(
            name='ReferencePaymentBatchFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(blank=True, db_index=True, default=django.utils.timezone.now, verbose_name='created')),
                ('file', models.FileField(upload_to='uploads', verbose_name='file')),
                ('errors', models.TextField(blank=True, default='', max_length=4086, verbose_name='errors')),
            ],
            options={
                'verbose_name': 'reference payment batch file',
                'verbose_name_plural': 'reference payment batch files',
            },
        ),
        migrations.RenameField(
            model_name='statement',
            old_name='statement_file',
            new_name='file',
        ),
        migrations.AddField(
            model_name='referencepaymentbatch',
            name='file',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='jbank.ReferencePaymentBatchFile'),
        ),
    ]



import logging
import subprocess
from decimal import Decimal
from os.path import join
from django.conf import settings
from django.core.management import CommandParser
from jbank.sepa import Pain001
from jutil.command import SafeCommand
from jutil.format import format_xml
from jutil.validators import iban_bic


logger = logging.getLogger(__name__)


class Command(SafeCommand):
    help = """
        Generates pain.001.001.03 compatible SEPA payment file.
        """

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('--verbose', action='store_true')
        parser.add_argument('--validate', action='store_true')

    def do(self, *args, **options):
        debtor_acc = 'FI4947300010416310'
        p = Pain001('201802071211XJANITEST', 'Kajala Group Oy', debtor_acc, iban_bic(debtor_acc), '020840699',
                    ['Koukkukankareentie 29', '20320 Turku'], 'FI')
        creditor_acc = 'FI8847304720017517'
        p.add_payment('201802071339A0001', 'Jani Kajala', creditor_acc, iban_bic(creditor_acc), Decimal('49.00'), 'vuokratilitys')
        xml_str = format_xml(p.render().decode())
        print(xml_str)

        filename = '/tmp/pain001.xml'
        with open(filename, 'wt') as fp:
            fp.write(xml_str)
            print(filename, 'written')

        if options['validate']:
            # /usr/bin/xmllint --format --pretty 1 --load-trace --debug --schema $1 $2
            res = subprocess.run([
                '/usr/bin/xmllint',
                '--format',
                '--pretty', '1',
                '--load-trace',
                '--debug',
                '--schema',
                join(settings.BASE_DIR, 'data/pain001/pain.001.001.03.xsd'),
                filename,
            ])
            if res.returncode == 0:
                print('OK')
            else:
                print('FAIL')



# Generated by Django 2.0 on 2018-02-09 05:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0019_auto_20180209_0437'),
    ]

    operations = [
        migrations.AddField(
            model_name='payout',
            name='due_date',
            field=models.DateField(blank=True, db_index=True, default=None, null=True, verbose_name='due date'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-11-02 22:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0003_statement_account'),
    ]

    operations = [
        migrations.CreateModel(
            name='StatementFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to='', verbose_name='file')),
            ],
            options={
                'verbose_name': 'account statement file',
                'verbose_name_plural': 'account statement files',
            },
        ),
    ]



# Generated by Django 2.0 on 2018-01-26 09:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0012_auto_20180126_0858'),
    ]

    operations = [
        migrations.AlterField(
            model_name='refundrecord',
            name='state',
            field=models.CharField(blank=True, choices=[('W', 'refund.waiting.payout'), ('P', 'refund.paid'), ('C', 'refund.canceled')], db_index=True, default='W', max_length=1, verbose_name='state'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-10-31 03:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('jacc', '0005_auto_20171030_1958'),
        ('jbank', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='referencepaymentbatch',
            name='filename',
        ),
        migrations.RemoveField(
            model_name='referencepaymentbatch',
            name='id',
        ),
        migrations.RemoveField(
            model_name='referencepaymentrecord',
            name='amount',
        ),
        migrations.RemoveField(
            model_name='referencepaymentrecord',
            name='id',
        ),
        migrations.RemoveField(
            model_name='statement',
            name='filename',
        ),
        migrations.RemoveField(
            model_name='statement',
            name='id',
        ),
        migrations.RemoveField(
            model_name='statementrecord',
            name='amount',
        ),
        migrations.RemoveField(
            model_name='statementrecord',
            name='id',
        ),
        migrations.AddField(
            model_name='referencepaymentbatch',
            name='accountentrysourcefile_ptr',
            field=models.OneToOneField(auto_created=True, default=None, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jacc.AccountEntrySourceFile'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='referencepaymentrecord',
            name='accountentry_ptr',
            field=models.OneToOneField(auto_created=True, default=None, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jacc.AccountEntry'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='statement',
            name='accountentrysourcefile_ptr',
            field=models.OneToOneField(auto_created=True, default=None, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jacc.AccountEntrySourceFile'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='statementrecord',
            name='accountentry_ptr',
            field=models.OneToOneField(auto_created=True, default=None, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jacc.AccountEntry'),
            preserve_default=False,
        ),
    ]



from pprint import pprint
from django.core.management import CommandParser
from jbank.parsers import parse_svm_batches_from_file
from jutil.command import SafeCommand


class Command(SafeCommand):
    help = 'Parses bank reference payment .SVM (saapuvat viitemaksut) files'

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('path', type=str)

    def do(self, *args, **options):
        batches = parse_svm_batches_from_file(options['path'])
        pprint(batches)



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-11-02 23:51
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0006_auto_20171102_2341'),
    ]

    operations = [
        migrations.AddField(
            model_name='statement',
            name='statement_file',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, to='jbank.StatementFile'),
        ),
        migrations.AlterField(
            model_name='statementfile',
            name='errors',
            field=models.TextField(blank=True, default='', max_length=4086, verbose_name='errors'),
        ),
    ]



# Generated by Django 2.0 on 2018-02-09 04:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0018_auto_20180208_1130'),
    ]

    operations = [
        migrations.AddField(
            model_name='payout',
            name='file_reference',
            field=models.CharField(blank=True, db_index=True, editable=False, max_length=255, verbose_name='file reference'),
        ),
        migrations.AddField(
            model_name='payoutstatus',
            name='response_code',
            field=models.CharField(blank=True, db_index=True, max_length=4, verbose_name='response code'),
        ),
        migrations.AddField(
            model_name='payoutstatus',
            name='response_text',
            field=models.CharField(blank=True, max_length=128, verbose_name='response text'),
        ),
        migrations.AlterField(
            model_name='payoutstatus',
            name='msg_id',
            field=models.CharField(blank=True, db_index=True, max_length=64, verbose_name='message id'),
        ),
        migrations.AlterField(
            model_name='payoutstatus',
            name='original_msg_id',
            field=models.CharField(blank=True, db_index=True, max_length=64, verbose_name='original message id'),
        ),
    ]



import base64
import logging
import os
from django.core.management import CommandParser
from jbank.helpers import process_pain002_file_content
from jbank.wsedi import wsedi_get
from jutil.command import SafeCommand


logger = logging.getLogger(__name__)


class Command(SafeCommand):
    help = """
        Download Finnish bank files. Assumes WS-EDI API parameter compatible HTTP REST API end-point.
        Uses project settings WSEDI_URL and WSEDI_TOKEN.
        """

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('path', type=str)
        parser.add_argument('--verbose', action='store_true')
        parser.add_argument('--overwrite', action='store_true')
        parser.add_argument('--file-type', type=str, help='E.g. TO, SVM, XP, NDCORPAYL, pain.002.001.03')
        parser.add_argument('--status', type=str, default='', help='E.g. DLD, NEW')
        parser.add_argument('--file-reference', type=str, help='Download single file based on file reference')
        parser.add_argument('--list-only', action='store_true')
        parser.add_argument('--process-pain002', action='store_true')

    def do(self, *args, **options):
        path = options['path']
        command = 'DownloadFileList'
        file_reference = options['file_reference']
        if file_reference:
            command = 'DownloadFile'
        status = options['status']
        file_type = options['file_type']
        if command == 'DownloadFileList' and not file_type:
            return print('--file-type required (e.g. TO, SVM, XP, NDCORPAYL, pain.002.001.03)')
        res = wsedi_get(command, file_type, status, file_reference, options['verbose'])
        if res.status_code >= 300:
            raise Exception("WS-EDI {} HTTP {}".format(command, res.status_code))
        data = res.json()
        """
            "FileDescriptors": {
                "FileDescriptor": [
                    {
                        "FileReference": "535283541",
                        "TargetId": "NONE",
                        "UserFilename": "STOL001.FMV80KT2.WEBSER.PS",
                        "ParentFileReference": "1218",
                        "FileType": "TO",
                        "FileTimestamp": "2017-12-18T20:33:09.362+02:00",
                        "Status": "DLD",
                        "LastDownloadTimestamp": "2017-12-19T12:36:34.490+02:00",
                        "ForwardedTimestamp": "2017-12-18T20:33:09.362+02:00",
                        "Deletable": "false",
                        "CustomerNumber": "06720106",
                        "Modifier": "06720106",
                        "ModifiedTimestamp": "2017-12-19T12:36:34.490+02:00",
                        "SourceId": "A",
                        "Environment": "PRODUCTION"
                    },
                    ...
        """
        if command == 'DownloadFileList':
            if 'FileDescriptors' in data and 'FileDescriptor' in data['FileDescriptors']:
                for fd in data['FileDescriptors']['FileDescriptor']:
                    file_reference = fd['FileReference']
                    file_type = fd['FileType']
                    file_basename = file_reference + '.' + file_type
                    file_path = os.path.join(path, file_basename)
                    if options['list_only']:
                        print('{file_reference} ({file_type}/{status}): {user_filename} ({timestamp})'.format(file_reference=file_reference, file_type=file_type, status=fd.get('Status'), user_filename=fd.get('UserFilename'), timestamp=fd.get('FileTimestamp')))
                        continue
                    if options['overwrite'] or not os.path.isfile(file_path):
                        command = 'DownloadFile'
                        res = wsedi_get(command, file_type, '', file_reference, options['verbose'])
                        if res.status_code >= 300:
                            raise Exception("WS-EDI {} HTTP {}".format(command, res.status_code))
                        file_data = res.json()
                        bcontent = base64.b64decode(file_data['Content'])
                        with open(file_path, 'wb') as fp:
                            fp.write(bcontent)
                        logger.info('Wrote file {}'.format(file_path))

                        # process selected files immediately
                        if options['process_pain002'] and file_type in ['XP', 'pain.002.001.03', 'NDCORPAYL']:
                            process_pain002_file_content(bcontent, file_path)
                    else:
                        print('Skipping old file {}'.format(file_path))
            else:
                print('Empty file list downloaded')
        elif command == 'DownloadFile':
            bcontent = base64.b64decode(data['Content'])
            file_path = os.path.join(path, file_reference)
            if options['overwrite'] or not os.path.isfile(file_path):
                with open(file_path, 'wb') as fp:
                    fp.write(bcontent)
                logger.info('Wrote file {}'.format(file_path))
            else:
                print('Skipping old file {}'.format(file_path))




from pprint import pprint
from django.core.management import CommandParser
from jbank.files import list_dir_files
from jbank.sepa import Pain002
from jutil.command import SafeCommand


class Command(SafeCommand):
    help = 'Parses pain.002 payment response .XP files'

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('path', type=str)
        parser.add_argument('--all', action='store_true')

    def do(self, *args, **options):
        files = list_dir_files(options['path'], '.XP')
        for f in files:
            print(f)
            with open(f, 'rb') as fp:
                p = Pain002(fp.read())
                print(p)



# Generated by Django 2.0 on 2018-02-08 06:26

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import jutil.validators


class Migration(migrations.Migration):

    dependencies = [
        ('jacc', '0010_auto_20180201_0734'),
        ('jbank', '0013_auto_20180126_0909'),
    ]

    operations = [
        migrations.CreateModel(
            name='Payout',
            fields=[
                ('accountentry_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='jacc.AccountEntry')),
                ('recipient_account_number', models.CharField(db_index=True, max_length=35, validators=[jutil.validators.iban_validator], verbose_name='recipient account number')),
                ('messages', models.TextField(blank=True, verbose_name='recipient messages')),
                ('msg_id', models.CharField(blank=True, db_index=True, editable=False, max_length=64, verbose_name='message id')),
                ('file_name', models.CharField(blank=True, db_index=True, editable=False, max_length=255, verbose_name='file name')),
                ('paid_date', models.DateTimeField(blank=True, db_index=True, default=None, null=True, verbose_name='paid date')),
                ('state', models.CharField(blank=True, choices=[('W', 'waiting'), ('P', 'paid'), ('C', 'canceled')], db_index=True, default='W', max_length=1, verbose_name='state')),
            ],
            options={
                'verbose_name_plural': 'payouts',
                'verbose_name': 'payout',
            },
            bases=('jacc.accountentry',),
        ),
        migrations.CreateModel(
            name='PayoutStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(blank=True, db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('file_name', models.CharField(blank=True, db_index=True, editable=False, max_length=255, verbose_name='file name')),
                ('msg_id', models.CharField(db_index=True, max_length=64, verbose_name='message id')),
                ('original_msg_id', models.CharField(db_index=True, max_length=64, verbose_name='original message id')),
                ('group_status', models.CharField(blank=True, db_index=True, max_length=8, verbose_name='group status')),
                ('status_reason', models.CharField(blank=True, max_length=255, verbose_name='status reason')),
                ('payout', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='jbank.Payout', verbose_name='payout')),
            ],
        ),
    ]
